<?php

const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHECK_STEP = 0;
const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_PRODUCT_STEP = 1;
const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_LOCATION_STEP = 2;
const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_TIME_STEP = 3;
const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_PREVIEW_STEP = 4;
const DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP = 5;

/**
 * Menu callback for change appointment form.
 */
function dvg_appointments_change_appointment() {
  /** @var \DvgAppointment $appointment */
  list($access, $appointment) = _dvg_appointments_change_appointment_parse_access_appointment();
  if (!$access) {
    drupal_not_found();
    return '';
  }
  return drupal_get_form('dvg_appointments_change_appointment_form', $appointment);
}

/**
 * @return array
 *   The access boolean and appointment if present.
 */
function _dvg_appointments_change_appointment_parse_access_appointment() {
  $params = drupal_get_query_parameters();
  $access = $appointment = FALSE;
  if (isset($params['id'], $params['token']) && is_numeric($params['id']) && !empty($params['token'])) {
    $appointment = dvg_appointments_load_appointment($params['id']);
    if ($appointment) {
      $access_token = dvg_appointments_access_appointment_token($appointment);
      if ($access_token === $params['token'] && $appointment->getStart()->getTimestamp() > time()) {
        $access = TRUE;
      }
    }
  }
  return array($access, $appointment);
}

/**
 * Change appointment form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 *
 * @return array
 *   Renderable array of the form.
 */
function dvg_appointments_change_appointment_form($form, &$form_state) {
  $appointment = $form_state['build_info']['args'][0];
  if (!isset($form_state['storage']['step'])) {
    $form_state['storage']['step'] = 0;
  }
  $step = $form_state['storage']['step'];
  $form['progressbar'] = _dvg_appointments_change_appointment_get_progressbar($step);

  $form['#attributes']['class'][] = 'appointment-webform';
  $node = node_load(functional_content_nid('dvg_appointments_create'));

  $next_step = t('Next page');
  switch ($step) {
    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHECK_STEP:
      $next_step = t('Change appointment');
      _dvg_appointments_change_appointment_check_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_PRODUCT_STEP:
      _dvg_appointments_change_appointment_product_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_LOCATION_STEP:
      _dvg_appointments_change_appointment_location_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_TIME_STEP:
      $next_step = $node->webform['preview_next_button_label'] ? t($node->webform['preview_next_button_label']) : t('Preview');
      _dvg_appointments_change_appointment_time_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_PREVIEW_STEP:
      $next_step = t('Submit');
      _dvg_appointments_change_appointment_preview_step($form, $form_state, $appointment);
      break;

    case DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP:
      // Before reaching here the user is redirected to the completed page.
      break;

    default:
      watchdog('dvg_appointments', 'Got unexpected step in appointment change form: @step', array('@step' => $step), WATCHDOG_ERROR);
      throw new InvalidArgumentException('Unexpected step encountered, see the Drupal logs.');
  }

  _dvg_appointments_change_appointment_form_add_actions($form, $step, $next_step);
  return $form;
}

/**
 * Creates a progressbar similar to webform.
 *
 * @param int $step
 *   The current step of the change appointment form.
 *
 * @return array
 *   A renderable progressbar.
 */
function _dvg_appointments_change_appointment_get_progressbar($step) {
  // Recycle create form for progressbar settings.
  $node = node_load(functional_content_nid('dvg_appointments_create'));
  // Page nums are not 0 based.
  $page_num = $step + 1;
  $has_locations = count(dvg_appointments_get_client_api()->get_gov_locations()) > 1;
  $page_labels = array();
  $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHECK_STEP] = t('Start');
  $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_PRODUCT_STEP] = t('Subject');
  if ($has_locations) {
    $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_LOCATION_STEP] = t('Location');
  }
  $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_TIME_STEP] = t('Date and time');
  $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_PREVIEW_STEP] = $node->webform['preview_title'] ? t($node->webform['preview_title']) : t('Preview');
  $page_labels[DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP] = t($node->webform['progressbar_label_confirmation']);
  $page_count = count($page_labels);
  // Correct page nums if there is no location since we are skipping a step.
  if (!$has_locations) {
    $page_labels = array_values($page_labels);
    if ($page_num > DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_LOCATION_STEP) {
      $page_num--;
    }
  }
  return array(
    '#theme' => 'webform_progressbar',
    '#node' => $node,
    '#page_num' => $page_num,
    '#page_count' => $page_count,
    '#page_labels' => $page_labels,
  );
}

/**
 * Adds the next/previous buttons to the change form.
 *
 * @param array $form
 *   Drupal form.
 * @param int $step
 *   Step number.
 * @param string $next_step
 *   Value of the "next" button.
 */
function _dvg_appointments_change_appointment_form_add_actions(&$form, $step, $next_step) {
  if ($step !== DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP) {
    $form['#submit'] = array('dvg_appointments_change_appointment_form_submit');
    $form['actions'] = array(
      '#type' => 'actions',
      '#weight' => 1000,
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $next_step,
      '#attributes' => array(
        'class' => array(
          'webform-next',
          'button-primary',
        ),
      ),
    );
    if ($step > DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHECK_STEP) {
      $form['actions']['previous'] = array(
        '#type' => 'submit',
        '#value' => t('Previous'),
        '#attributes' => array(
          'class' => array(
            'webform-previous',
          ),
        ),
      );
    }
  }
}

/**
 * Change appointment form submit handler.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 */
function dvg_appointments_change_appointment_form_submit(&$form, &$form_state) {
  $step = $form_state['storage']['step'];
  if ($form_state['triggering_element']['#value'] === t('Previous')) {
    $step--;
  }
  else {
    $step++;
  }
  // Skip location step if there is only 1 location.
  if (
    $step === DVG_APPOINTMENTS_CHANGE_APPOINTMENT_CHANGE_LOCATION_STEP
    && count(dvg_appointments_get_client_api()->get_gov_locations()) <= 1
  ) {
    $step++;
  }
  $form_state['storage']['step'] = $step;

  if ($step !== DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP) {
    $form_state['rebuild'] = TRUE;
  }

  // Store all submitted data in storage.
  foreach ($form_state['values'] as $key => $value) {
    if (!in_array($key, array('submit', 'previous', 'form_build_id', 'form_token', 'form_id', 'op'), TRUE)) {
      $form_state['storage'][$key] = $value;
    }
  }
}

/**
 * Initial page of the appointment change form.
 *
 * Shows the current appointment.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_change_appointment_check_step(&$form, &$form_state, $appointment) {
  $form['appointment'] = array(
    '#markup' => _dvg_appointments_table($appointment)
  );
}

/**
 * Product selection page of the appointment change form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_change_appointment_product_step(&$form, &$form_state, $appointment) {
  $current_products = array();
  if (empty($form_state['storage']['products'])) {
    foreach ($appointment->getProductIds() as $product_id) {
      $current_products[] = array(
        'product' => $product_id,
        'count' => $appointment->getProductCount($product_id),
      );
    }
  }
  else {
    $current_products = $form_state['storage']['products'];
  }

  // Add our own validation and disable Drupal validation handling, since we
  // allow adding extra products with JavaScript.
  $form['#validate'][] = '_dvg_appointments_change_appointment_product_validate';
  $form['products'] = array(
    '#type' => 'container',
    '#title' => t('Subject'),
    '#tree' => TRUE,
    '#id' => 'appointments-change-products',
    '#validated' => TRUE,
  );
  foreach ($current_products as $product) {
    $form['products'][] = array(
      '#type' => 'wa_product',
      '#title' => '',
      '#required' => TRUE,
      '#default_value' => $product,
      '#webform_component' => array(
        'form_key' => 'product',
      ),
      '#attributes' => array(
        'class' => array(
          'appointment-change-product',
        ),
      ),
      '#validated' => TRUE,
      '#prefix' => '<div class="wfm-item">',
      '#suffix' => '</div>',
    );
  }

  $create_appointment_node = node_load(functional_content_nid('dvg_appointments_create'));
  $product_cid = dvg_appointments_get_cid($create_appointment_node, 'wa_product');
  if (module_exists('wfm') && _wfm_is_multiple($create_appointment_node->webform['components'][$product_cid])) {
    // Add multiple product support.
    $form['#attached']['js'][] = drupal_get_path('module', 'dvg_appointments') . '/js/dvg_appointments.change.js';
  }
}

/**
 * Validate handler for the product selection step.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 */
function _dvg_appointments_change_appointment_product_validate($form, &$form_state) {
  $client_api = dvg_appointments_get_client_api();
  $available_products = $client_api->get_available_products();
  if (!isset($form_state['input']['products'])) {
    form_set_error('products', t('You must select at least one product.'));
  }
  $form_state['input']['products'] = array_values($form_state['input']['products']);
  $valid_products = 0;
  foreach ($form_state['input']['products'] as $index => $input) {
    if (empty($input['product']) && empty($input['count'])) {
      unset($form_state['input']['products'][$index], $form_state['values']['products'][$index]);
      continue;
    }
    if (empty($input['product']) || empty($input['count'])) {
      form_set_error("products[$index]", t('Please select both a product and set the number of persons.'));
      continue;
    }
    if (!is_numeric($input['count'])) {
      form_set_error("products[$index]", t('Please use only numbers for person count.'));
      continue;
    }
    if (!isset($available_products[$input['product']])) {
      form_set_error("products[$index]", t('Unknown product selected, please only use the dropdown.'));
      continue;
    }

    $valid_products++;
    $form_state['values']['products'][$index] = $form_state['input']['products'][$index];
  }
  $form_state['values']['products'] = array_values($form_state['values']['products']);
  if ($valid_products === 0) {
    form_set_error('products', t('Please select at least one product.'));
  }
}

/**
 * Location selection page of the appointment change form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_change_appointment_location_step(&$form, &$form_state, $appointment) {
  if (isset($form_state['storage']['location'])) {
    $default_value = $form_state['storage']['location'];
  }
  else {
    $default_value = $appointment->getLocation();
  }
  $form['location'] = array(
    '#type' => 'wa_location',
    '#title' => t('Location'),
    '#default_value' => $default_value,
    '#required' => TRUE,
  );
}

/**
 * Date time selection page of the appointment change form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_change_appointment_time_step(&$form, &$form_state, $appointment) {
  // Not restoring the default value from the original appointment. That slot is
  // already taken by the original appointment. Users must always pick a
  // new slot.
  $form['date_time'] = array(
    '#type' => 'wa_date_time',
    '#title' => t('Date and time'),
    '#required' => TRUE,
    '#tree' => TRUE,
    // This component expects a default value to be set.
    '#default_value' => NULL,
  );
  if (isset($form_state['storage']['date_time'])) {
    $form['date_time']['#default_value'] = $form_state['storage']['date_time'];
  }
}

/**
 * Preview page of the appointment change form.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 * @param \DvgAppointment $appointment
 *   The appointment that is being changed.
 */
function _dvg_appointments_change_appointment_preview_step(&$form, &$form_state, $appointment) {
  $preview_appointment = new DvgAppointment();

  $client_api = dvg_appointments_get_client_api();
  $available_products = $client_api->get_available_products();
  foreach ($form_state['storage']['products'] as $product) {
    $product_details = $available_products[$product['product']];
    $additional_customer_duration = isset($product_details['additional_customer_duration']) ? $product_details['additional_customer_duration'] : $product_details['duration'];
    $preview_appointment->addProduct($product['product'], $product['count'], $product_details['duration'] * 60, $additional_customer_duration * 60);
  }
  $duration = $preview_appointment->getAppointmentDurationEstimate();

  $start_time = $form_state['storage']['date_time'][0];
  $preview_appointment->setStart(DateTime::createFromFormat('U', $start_time));
  $preview_appointment->setEnd(DateTime::createFromFormat('U', $start_time + $duration));

  if (isset($form_state['storage']['location'])) {
    $preview_appointment->setLocation($form_state['storage']['location']);
  }

  $form['preview_title'] = array(
    '#markup' => '<h2>' . t('Check and confirm your changed appointment') . '</h2>',
  );
  $form['appointment_preview'] = array(
    '#markup' => _dvg_appointments_table($preview_appointment),
  );

  $form['#validate'][] = 'dvg_appointments_book_changed_appointment';
}

/**
 * Cancels the old appointment, books the new one.
 *
 * @param array $form
 *   Drupal form.
 * @param array $form_state
 *   The current state of the form.
 *
 * @see dvg_appointments_book_appointment().
 */
function dvg_appointments_book_changed_appointment(&$form, &$form_state) {
  if ($form_state['triggering_element']['#value'] === t('Previous')) {
    return;
  }
  /** @var \DvgAppointment $old_appointment */
  $old_appointment = $form_state['build_info']['args'][0];

  // Booking an appointment requires arrays instead of the DvgAppointment so do
  // some conversion.
  $appointment_data = array(
    'datetime' => $form_state['storage']['date_time'][0],
    'products' => dvg_appointments_get_selected_products($form_state, $form),
  );

  $client_api = dvg_appointments_get_client_api();
  $location = FALSE;
  if (isset($form_state['storage']['location'])) {
    $location = $form_state['storage']['location'];
    $client_api->set_location($location);
  }

  $create_appointment_node = node_load(functional_content_nid('dvg_appointments_create'));
  $submission = webform_menu_submission_load($old_appointment->getSubmissionId(), $old_appointment->getNodeId());
  $mapping = unserialize($create_appointment_node->webform['appointment_mapping']);
  $book_data = array();
  foreach ($mapping as $field => $cid) {
    $book_data[$field] = '';
    if (isset($submission->data[$cid][0])) {
      $book_data[$field] = $submission->data[$cid][0];
    }
  }

  $new_appointment = $client_api->book_appointment($appointment_data, $book_data);

  // Replace the old appointment.
  if ($location) {
    $new_appointment->setLocation($location);
  }
  $new_appointment->setId($old_appointment->getId());
  $new_appointment->setNodeId($old_appointment->getNodeId());
  $new_appointment->setSubmissionId($old_appointment->getSubmissionId());

  $client_api->delete_appointment($old_appointment);
  dvg_appointments_store_appointment($new_appointment);

  // Update the submission if necessary.
  $is_changed = FALSE;
  $product_cid = dvg_appointments_get_cid($create_appointment_node, 'wa_product');
  if ($form_state['storage']['products'] !== $submission->data[$product_cid]) {
    $is_changed = TRUE;
    $submission->data[$product_cid] = $form_state['storage']['products'];
  }
  $date_time_cid = dvg_appointments_get_cid($create_appointment_node, 'wa_date_time');
  if ($form_state['storage']['date_time'][0] !== $submission->data[$date_time_cid][0]) {
    $is_changed = TRUE;
    $submission->data[$date_time_cid][0] = $form_state['storage']['date_time'][0];
  }
  if ($location) {
    $location_cid = dvg_appointments_get_cid($create_appointment_node, 'wa_location');
    if ($form_state['storage']['location'] !== $submission->data[$location_cid][0]) {
      $is_changed = TRUE;
      $submission->data[$location_cid][0] = $form_state['storage']['location'];
    }
  }

  if ($is_changed) {
    webform_submission_update($create_appointment_node, $submission);
  }

  if (module_exists('webform_encrypt')) {
    // Decrypt submission again after update.
    webform_encrypt_webform_submission_load(array($submission));
  }

  // Resend mails.
  webform_submission_send_mail($create_appointment_node, $submission, $create_appointment_node->webform['emails']);

  $link_options = array('query' => array(
    'id' => $new_appointment->getId(),
    'token' => dvg_appointments_access_appointment_token($new_appointment),
  ));
  drupal_goto(_dvg_appointments_get_change_form_url() . '/done', $link_options);
}

/**
 * Menu callback for change appointment done step.
 *
 * @return array|string
 *   Renderable page.
 */
function dvg_appointments_change_appointment_finished_step() {
  /** @var \DvgAppointment $new_appointment */
  list($access, $new_appointment) = _dvg_appointments_change_appointment_parse_access_appointment();
  if (!$access) {
    drupal_not_found();
    return '';
  }

  // Fake the progress bar to act "completed".
  $page['progressbar'] = _dvg_appointments_change_appointment_get_progressbar(DVG_APPOINTMENTS_CHANGE_APPOINTMENT_FINISHED_STEP);

  $page['appointment_complete'] = array(
    '#markup' => _dvg_appointments_create_info($new_appointment),
  );

  $link_options = array('query' => array(
    'id' => $new_appointment->getId(),
    'token' => dvg_appointments_access_appointment_token($new_appointment),
  ));
  $change_link = l(t('Change appointment'), _dvg_appointments_get_change_form_url(), $link_options);
  $completed_args = array(
    '!change_link' => $change_link,
  );
  $cancel_nid = functional_content_nid('dvg_appointments_cancel');
  if ($cancel_nid) {
    $cancel_link = l(t('Cancel appointment'), 'node/' . $cancel_nid);
    $completed_args['!cancel_link'] = $cancel_link;
    $completed_message = t('Use the following links to change or cancel your appointment:<br>!change_link<br>!cancel_link', $completed_args);
  }
  else {
    $completed_message = t('Use the following link to change your appointment:<br>!change_link', $completed_args);
  }
  $page['appointment_action_urls'] = array(
    '#markup' => '<p class="appointment-action-urls">' . $completed_message . '</p>',
  );

  return $page;
}
